@@ -395,18 +395,12 @@ int zoomSurfaceRGBA(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy,
 
 int zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 {
-    Uint32 x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, sstep;
+    Uint32 x, y, sx, sy, *sax, *say, *csax, *csay;
+    int csx, csy;
     Uint8 *sp, *dp, *csp;
     int dgap;
 
     /*
-     * Variable setup 
-     */
-    sx = (Uint32) (65536.0 * (float) src->w / (float) dst->w);
-    sy = (Uint32) (65536.0 * (float) src->h / (float) dst->h);
-
- 
-     /*
      * Allocate memory for row increments 
      */
     if ((sax = (Uint32 *) malloc((dst->w + 1) * sizeof(Uint32))) == NULL) {
@@ -432,19 +426,25 @@ int zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
      */
     csx = 0;
     csax = sax;
-    for (x = 0; x <= dst->w; x++) {
-	*csax = csx;
+    for (x = 0; x < dst->w; x++) {
+	csx += src->w;
+	*csax = 0;
+	while (csx >= dst->w) {
+	    csx -= dst->w;
+	    (*csax)++;
+	}
 	csax++;
-	csx &= 0xffff;
-	csx += sx;
     }
     csy = 0;
     csay = say;
-    for (y = 0; y <= dst->h; y++) {
-	*csay = csy;
+    for (y = 0; y < dst->h; y++) {
+	csy += src->h;
+	*csay = 0;
+	while (csy >= dst->h) {
+	  csy -= dst->h;
+	  (*csay)++;
+	}
 	csay++;
-	csy &= 0xffff;
-	csy += sy;
     }
 
 
@@ -463,10 +463,8 @@ int zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 	    /*
 	     * Advance source pointers 
 	     */
+	    sp += (*csax) * (flipx ? -1 : 1);
 	    csax++;
-            sstep = (*csax >> 16);
-            if (flipx) sstep = -sstep;
-            sp += sstep;
 	    /*
 	     * Advance destination pointer 
 	     */
@@ -475,10 +473,8 @@ int zoomSurfaceY(SDL_Surface * src, SDL_Surface * dst, int flipx, int flipy)
 	/*
 	 * Advance source pointer (for row) 
 	 */
+	csp += ((*csay) * src->pitch) * (flipy ? -1 : 1);
 	csay++;
-        sstep = (*csay >> 16) * src->pitch;
-        if (flipy) sstep = -sstep;
-        csp = ((Uint8 *) csp + sstep);
 
 	/*
 	 * Advance destination pointers 
