@@ -3,6 +3,8 @@
  TestGfxPrimitives
  
  Test graphics primitive routines for accuracy (visual) and speed.
+
+ Copyright (C) A. Schiffler, June 2001, GPL
  
 */
 
@@ -14,7 +16,7 @@
 
 #include "SDL.h"
 
-#include "SDL_gfxPrimitives.h"
+#include "SDL/SDL_gfxPrimitives.h"
 
 #define WIDTH	640
 #define HEIGHT	480
@@ -1262,6 +1264,86 @@ void BenchmarkFilledEllipse(SDL_Surface *screen)
  stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
 }
 
+void TestArc(SDL_Surface *screen)
+{
+ int i;
+ char r,g,b;
+ 
+ /* Create random points */
+ srand(time(NULL));
+ InitRandomPoints();
+ 
+ /* Draw A=255 */
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  arcRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ for (i=0; i<NUM_RANDOM; i++) {
+  arcRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,80+(HEIGHT-80)/2,WIDTH,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  arcRGBA(screen, rx2[i], ry2[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+ }
+
+ /* Draw Colortest */
+ SetClip(screen,0,80+(HEIGHT-80)/2,WIDTH/2,HEIGHT);
+ for (i=0; i<NUM_RANDOM; i++) {
+  if (rx1[i] < (WIDTH/6))  {
+   r=255; g=0; b=0; 
+  } else if (rx1[i] < (WIDTH/3) ) {
+   r=0; g=255; b=0; 
+  } else {
+   r=0; g=0; b=255; 
+  }
+  arcRGBA(screen, rx1[i], ry2[i], rr1[i], a1[i], a2[i], r, g, b, 255);
+ }
+}
+
+void BenchmarkArc(SDL_Surface *screen)
+{
+ int i,j;
+ int repeat;
+ Uint32 time1, time2;
+ char titletext[256];
+ 
+ /* Draw A=255 */
+ repeat=50;
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   arcRGBA(screen, rx1[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], 255);
+  }
+ }
+ time2=SDL_GetTicks();
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+ 
+ /* Draw A=various */
+ repeat=5;
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<NUM_RANDOM; i++) {
+   arcRGBA(screen, rx2[i], ry1[i], rr1[i], a1[i], a2[i], rr[i], rg[i], rb[i], ra[i]);
+  }
+ }
+ time2=SDL_GetTicks();
+
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(NUM_RANDOM*repeat)/(float)(time2-time1));
+ stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+}
+
 void TestPie(SDL_Surface *screen)
 {
  int i;
@@ -1687,6 +1769,97 @@ void BenchmarkFilledPolygon(SDL_Surface *screen)
  stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
 }
 
+void TestTexturedPolygon(SDL_Surface *screen)
+{
+ int i;
+ 
+ /* Create random points */
+ srand(time(NULL));
+ InitRandomPoints();
+ SDL_Surface *texture;
+
+ /* Load texture surface */
+ texture = SDL_LoadBMP("texture.bmp");
+ 
+ /* Draw A=255 */
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+  texturedPolygon(screen, &rx1[i], &ry1[i], 3, texture, rr1[i], rr2[i]);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+  SDL_SetAlpha(texture, SDL_SRCALPHA, ra[i]);
+  texturedPolygon(screen, &rx2[i], &ry1[i], 3, texture, rr1[i], rr2[i]);
+ }
+
+ /* Draw A=various */
+ SetClip(screen,WIDTH/2,80+(HEIGHT-80)/2,WIDTH,HEIGHT);
+ for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+  texturedPolygon(screen, &rx2[i], &ry2[i], 3, texture, rr1[i], rr2[i]);
+ }
+
+ /* Draw Colortest */
+ SetClip(screen,0,80+(HEIGHT-80)/2,WIDTH/2,HEIGHT);
+ for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+  rx1[i+1]=rx1[i]+rr1[i];
+  rx1[i+2]=rx1[i];
+  ry2[i+1]=ry2[i];
+  ry2[i+2]=ry2[i]+rr2[i];
+  texturedPolygon(screen, &rx1[i], &ry2[i], 3, texture, 0, 0);
+ }
+
+}
+
+void BenchmarkTexturedPolygon(SDL_Surface *screen)
+{
+ int i,j;
+ int repeat;
+ Uint32 time1, time2;
+ char titletext[256];
+ SDL_Surface *texture;
+
+ /* Load texture surface */
+ texture = SDL_LoadBMP("texture.bmp");
+
+ /* Create random points */
+ srand(time(NULL));
+ InitRandomPoints();
+ 
+ /* Draw A=255 */
+ repeat=20;
+ SetClip(screen,0,60,WIDTH/2,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+   texturedPolygon(screen, &rx1[i], &ry1[i], 3, texture, rr1[i], rr2[i]);
+  }
+ }
+ time2=SDL_GetTicks();
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(((NUM_RANDOM/2-3)/3)*repeat)/(float)(time2-time1));
+ stringRGBA (screen, WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+ 
+ /* Draw A=various */
+ repeat=5;
+ SetClip(screen,WIDTH/2,60,WIDTH,60+(HEIGHT-80)/2);
+ time1=SDL_GetTicks();
+ for (j=0; j<repeat; j++) {
+  for (i=0; i<(NUM_RANDOM/2-3); i += 3) {
+   SDL_SetAlpha(texture, SDL_SRCALPHA, ra[i]);
+   texturedPolygon(screen, &rx2[i], &ry1[i], 3, texture, rr1[i], rr2[i]);
+  }
+ }
+ time2=SDL_GetTicks();
+
+ /* Results */
+ SetClip(screen,0,0,WIDTH-1,HEIGHT-1);
+ sprintf (titletext, "%.0f per sec",1000.0*(float)(((NUM_RANDOM/2-3)/3)*repeat)/(float)(time2-time1));
+ stringRGBA (screen, 3*WIDTH/4-4*strlen(titletext),30-4,titletext,255,255,255,255);
+}
+
 void TestBezier(SDL_Surface *screen)
 {
  int i;
@@ -1786,6 +1959,10 @@ int main(int argc, char *argv[])
 	int done;
 	SDL_Event event;
 	int oldprim, curprim;
+        char title[64];
+        
+        /* Generate title string */
+        sprintf (title,"TestGfxPrimitives - Ver. %i.%i.%i",SDL_GFXPRIMITIVES_MAJOR, SDL_GFXPRIMITIVES_MINOR, SDL_GFXPRIMITIVES_MICRO);
         
 	/* Initialize SDL */
 	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
@@ -1836,7 +2013,7 @@ int main(int argc, char *argv[])
 	SDL_SetAlpha(screen, SDL_SRCALPHA, 0);
 	
 	/* Set title for window */
- 	SDL_WM_SetCaption("TestGfxPrimitives","TestGfxPrimitives");
+ 	SDL_WM_SetCaption(title,title);
  	
 	/* Wait for a keystroke */
 	done = 0;
@@ -1978,8 +2155,17 @@ int main(int argc, char *argv[])
 	 		  oldprim=curprim; 
 			  break;
 
-			 /* --- Pie */
+			 /* --- Arc */
 			 case 15:
+		 	  ClearScreen(screen, "Arc");
+			  TestArc(screen);
+			  BenchmarkArc(screen);
+ 			  /* Next primitive */ 			  
+	 		  oldprim=curprim; 
+			  break;
+			  
+			 /* --- Pie */
+			 case 16:
 		 	  ClearScreen(screen, "Pie");
 			  TestPie(screen);
 			  BenchmarkPie(screen);
@@ -1988,7 +2174,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- Filled Pie */
-			 case 16:
+			 case 17:
 		 	  ClearScreen(screen, "Filled Pie");
 			  TestFilledPie(screen);
 			  BenchmarkFilledPie(screen);
@@ -1997,7 +2183,7 @@ int main(int argc, char *argv[])
 			  break;
 
 			 /* --- Polygon */
-			 case 17:
+			 case 18:
 		 	  ClearScreen(screen, "Polygon");
 			  TestPolygon(screen);
 			  BenchmarkPolygon(screen);
@@ -2006,7 +2192,7 @@ int main(int argc, char *argv[])
 			  break;
 			 
 			 /* --- AA-Polygon */
-			 case 18:
+			 case 19:
 		 	  ClearScreen(screen, "AA-Polygon");
 			  TestAAPolygon(screen);
 			  BenchmarkAAPolygon(screen);
@@ -2015,7 +2201,7 @@ int main(int argc, char *argv[])
 			  break;
 			 
 			 /* ---- Filled Polygon */ 
-			 case 19:
+			 case 20:
 		 	  ClearScreen(screen, "Filled Polygon");
 			  TestFilledPolygon(screen);
 			  BenchmarkFilledPolygon(screen);
@@ -2023,8 +2209,17 @@ int main(int argc, char *argv[])
 	 		  oldprim=curprim; 
 			  break;
 
+			 /* ---- Textured Polygon */ 
+			 case 21:
+		 	  ClearScreen(screen, "Textured Polygon");
+			  TestTexturedPolygon(screen);
+			  BenchmarkTexturedPolygon(screen);
+ 			  /* Next primitive */ 			  
+	 		  oldprim=curprim; 
+			  break;
+
 			 /* ---- Bezier Curve */ 
-			 case 20:
+			 case 22:
 		 	  ClearScreen(screen, "Bezier Curve");
 			  TestBezier(screen);
 			  BenchmarkBezier(screen);
@@ -2041,6 +2236,7 @@ int main(int argc, char *argv[])
 			
 			/* Show */
 			SDL_UpdateRect(screen, 0, 0, 0, 0);
+			SDL_Flip(screen);
 		}
 		
 		/* Slow down polling */
